@@ -15,13 +15,13 @@ def load_file_to_dict(path):
     Takes a file path and loads it into a nested json dict following the format in json_form_schema.json
     The file may be a xls file or json file. If it is xls it is converted using xls2json.
     """
-    if path.endswith(".xls") or path.endswith(".csv"):
+    if path.endswith(".json"):
+        name = _section_name(path)
+        return (name, utils.get_pyobj_from_json(path))
+    else:
         name = _section_name(path)
         excel_reader = SurveyReader(path)
         return (name, excel_reader.to_json_dict())
-    elif path.endswith(".json"):
-        name = _section_name(path)
-        return (name, utils.get_pyobj_from_json(path))
 
 def collect_compatible_files_in_directory(directory):
     """
