@@ -10,6 +10,7 @@ import constants
 from errors import PyXFormError
 from xls2json_backends import xls_to_dict, csv_to_dict, get_cascading_json
 from utils import is_valid_xml_tag
+import datetime
 
 ####### STATIC DATA #######
 
@@ -70,7 +71,9 @@ survey_header_aliases = {
     u"image": u"media::image",
     u"audio": u"media::audio",
     u"video": u"media::video",
-    u"count": u"bind::jr:count"
+    u"count": u"control::jr:count",
+    u"repeat_count": u"control::jr:count",
+    u"jr:count": u"control::jr:count",
 }
 list_header_aliases = {
     u"caption" : constants.LABEL,
@@ -90,18 +93,20 @@ type_aliases = {
     u"add video prompt" : u"video"
 }
 yes_no_aliases = {
-    "yes": "true()",
-    "Yes": "true()",
-    "YES": "true()",
-    "true": "true()",
-    "True": "true()",
-    "TRUE": "true()",
-    "no": "false()",
-    "No": "false()",
-    "NO": "false()",
-    "false": "false()",
-    "False": "false()",
-    "FALSE": "false()"
+    "yes": True,
+    "Yes": True,
+    "YES": True,
+    "true": True,
+    "True": True,
+    "TRUE": True,
+    "true()": True,
+    "no": False,
+    "No": False,
+    "NO": False,
+    "false": False,
+    "False": False,
+    "FALSE": False,
+    "false()": False,
 }
 label_optional_types = [
     u"deviceid",
@@ -216,15 +221,16 @@ def dealias_types(dict_array):
             row[constants.TYPE] = type_aliases[found_type]
     return dict_array
 
-def clean_unicode_values(dict_array):
+def clean_text_values(dict_array):
     """
-    Go though the dict array and removing double spaces and trailing and leading spaces from all unicode values
+    Go though the dict array and strips all text values.
+    Also replaces multiple spaces with single spaces.
     Note that the keys don't get cleaned, which could be an issue.
     """
     for row in dict_array:
         for key, value in row.items():
-            if type(value) is unicode:
-                row[key] = re.sub(r"\s+", " ", value.strip())
+            if isinstance(value, basestring):
+                row[key] = re.sub(r"( )+", " ", value.strip())
     return dict_array
 
 #This is currently unused because name uniqueness is checked in the json2xform code.
@@ -295,6 +301,8 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
         #Set warnings to a list that will be discarded.
         warnings = []
     
+    rowFormatString = '[row : %s]'
+    
     #Make sure the passed in vars are unicode
     form_name = unicode(form_name)
     default_language = unicode(default_language)
@@ -314,7 +322,7 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
     #add_none_option is a boolean that when true, indicates a none option should automatically be added to selects.
     #It should probably be deprecated but I haven't checked yet.
     if u"add_none_option" in settings:
-        settings[u"add_none_option"] = yes_no_aliases.get(settings[u"add_none_option"], u"false()") == u"true()"
+        settings[u"add_none_option"] = yes_no_aliases.get(settings[u"add_none_option"], False)
     
     #Here we create our json dict root with default settings:
     id_string = settings.get(constants.ID_STRING, form_name)
@@ -324,6 +332,9 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
        constants.TITLE : id_string,
        constants.ID_STRING : id_string,
        constants.DEFAULT_LANGUAGE : default_language,
+       #By default the version is based on the date and time yyyymmddhh
+       #Leaving default version out for now since it might cause problems for formhub.
+       #constants.VERSION : datetime.datetime.now().strftime("%Y%m%d%H"), 
        constants.CHILDREN : []
     }
     #Here the default settings are overridden by those in the settings sheet
@@ -339,21 +350,33 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
     
     choices_sheet = workbook_dict.get(constants.CHOICES, [])
     choices_sheet = dealias_and_group_headers(choices_sheet, list_header_aliases, use_double_colons, default_language)
-    
+    ########### Cascading Select sheet ###########
+    cascading_choices = workbook_dict.get(constants.CASCADING_CHOICES, [])
+    if len(cascading_choices):
+        if cascading_choices[0].has_key('choices'):
+            choices_sheet = choices_sheet + cascading_choices[0]['choices']
+
     combined_lists = group_dictionaries_by_key(choices_and_columns_sheet + choices_sheet + columns_sheet, constants.LIST_NAME)
     
-                
     choices = combined_lists
+    #Make sure all the options have the required properties:
+    for list_name, options in choices.items():
+        for option in options:
+            if 'name' not in option:
+                info = "[list_name : " + list_name + ']'
+                raise PyXFormError("On the choices sheet there is a option with no name. " + info)
+            if 'label' not in option:
+                info = "[list_name : " + list_name + ']'
+                warnings.append("On the choices sheet there is a option with no label. " + info)
 
-    ########### Cascading Select sheet ###########
-    cascading_choices = workbook_dict.get(constants.CASCADING_CHOICES, {})
-    
     ########### Survey sheet ###########
     if constants.SURVEY not in workbook_dict:
         raise PyXFormError("You must have a sheet named (case-sensitive): " + constants.SURVEY)
     survey_sheet = workbook_dict[constants.SURVEY]
     #Process the headers:
-    survey_sheet = clean_unicode_values(survey_sheet)
+    clean_text_values_enabled = yes_no_aliases.get(settings.get("clean_text_values", "true()"))
+    if clean_text_values_enabled:
+        survey_sheet = clean_text_values(survey_sheet)
     survey_sheet = dealias_and_group_headers(survey_sheet, survey_header_aliases, use_double_colons, default_language)
     survey_sheet = dealias_types(survey_sheet)
     ##################################
@@ -365,7 +388,6 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
     stack = [(None, json_dict.get(constants.CHILDREN))]
     #If a group has a table-list appearance flag this will be set to the name of the list
     table_list = None
-    begin_table_list = False
     #For efficiency we compile all the regular expressions that will be used to parse types:
     end_control_regex = re.compile(r"^(?P<end>end)(\s|_)(?P<type>("
                                    + '|'.join(control_aliases.keys()) + r"))$")
@@ -381,27 +403,27 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
     for row in survey_sheet:
         row_number += 1
         prev_control_type, parent_children_array = stack[-1]
-        
         #Disabled should probably be first so the attributes below can be disabled.
         if u"disabled" in row:
-            warnings.append("The 'disabled' column header is not part of the current spec. We recommend using relevant instead.")
+            warnings.append(rowFormatString % row_number +
+                            " The 'disabled' column header is not part of the current spec." +
+                            "We recommend using relevant instead.")
             disabled = row.pop(u"disabled")
-            if disabled in yes_no_aliases:
-                disabled = yes_no_aliases[disabled]
-            if disabled == 'true()':
+            if yes_no_aliases.get(disabled):
                 continue
         
         #skip empty rows
         if len(row) == 0: continue
-        
+
         #Get question type
         question_type = row.get(constants.TYPE)
         if not question_type:
             # if name and label are also missing, then its a comment row, and we skip it with warning
             if not ((constants.NAME in row) and (constants.LABEL in row)):
-                    warnings.append("Row wihtout name, text, or label is being skipped " + str(row_number) + ": " + str(row))
+                    warnings.append(rowFormatString % row_number +
+                        " Row without name, text, or label is being skipped:\n" + str(row))
                     continue
-            raise PyXFormError("Question with no type on row " + str(row_number))
+            raise PyXFormError(rowFormatString % row_number + " Question with no type.")
             continue
         
         #Check if the question is actually a setting specified on the survey sheet
@@ -417,18 +439,29 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
             if parse_dict.get("end") and "type" in parse_dict:
                 control_type = control_aliases[parse_dict["type"]]
                 if prev_control_type != control_type or len(stack) == 1:
-                    raise PyXFormError("Unmatched end statement. Previous control type: " + str(prev_control_type) + ", Control type: " + str(control_type))
+                    raise PyXFormError(rowFormatString % row_number +
+                                       " Unmatched end statement. Previous control type: " +
+                                       str(prev_control_type) +
+                                       ", Control type: " + str(control_type))
                 stack.pop()
                 table_list = None
                 continue
         
-        #Make sure the question has a valid name
-        question_name = unicode(row.get(constants.NAME))
-        if not question_name:
-            raise PyXFormError("Question with no name on row " + str(row_number))
+        #Make sure the row has a valid name
+        if not constants.NAME in row:
+            #TODO: It could be slick if had nameless groups generate a flat model
+            #      with only a body element.
+            if row['type'] == 'note':
+                #autogenerate names for notes without them
+                row['name'] = "generated_note_name_" + str(row_number)
+            else:
+                raise PyXFormError(rowFormatString % row_number + " Question or group with no name.")
+        question_name = unicode(row[constants.NAME])
         if not is_valid_xml_tag(question_name):
-            error_message = "Invalid question name [" + question_name + "] on row " + str(row_number) + "\n"
-            error_message += "Names must begin with a letter, colon, or underscore. Subsequent characters can include numbers, dashes, and periods."
+            error_message =  rowFormatString % row_number
+            error_message += " Invalid question name [" + question_name + "]"
+            error_message += "Names must begin with a letter, colon, or underscore."
+            error_message += "Subsequent characters can include numbers, dashes, and periods."
             raise PyXFormError(error_message)
         
         if constants.LABEL not in row and \
@@ -437,9 +470,9 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
             #TODO: Should there be a default label?
             #      Not sure if we should throw warnings for groups...
             #      Warnings can be ignored so I'm not too concerned about false positives.
-            warnings.append("Warning unlabeled question in row " + str(row_number) + ": " + str(row))
+            warnings.append(rowFormatString % row_number + " Question has no label: " +  str(row))
         
-        #Try to parse question as begin control statement (i.e. begin loop/repeat/group:
+        #Try to parse question as begin control statement (i.e. begin loop/repeat/group):
         begin_control_parse = begin_control_regex.search(question_type)
         if begin_control_parse:
             parse_dict = begin_control_parse.groupdict()
@@ -455,16 +488,31 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
                 if control_type is constants.LOOP:
                     if not parse_dict.get("list_name"):
                         #TODO: Perhaps warn and make repeat into a group?
-                        raise PyXFormError("Repeat without list name " + " Error on row: " + str(row_number))
+                        raise PyXFormError(rowFormatString % row_number + " Repeat loop without list name.")
                     list_name = parse_dict["list_name"]
                     if list_name not in choices:
-                        raise PyXFormError("List name not in columns sheet: " + list_name + " Error on row: " + str(row_number))
+                        raise PyXFormError(rowFormatString % row_number + " List name not in columns sheet: " + list_name)
                     new_json_dict[constants.COLUMNS] = choices[list_name]
                 
                 #Code to deal with table_list appearance flags (for groups of selects)
                 if new_json_dict.get(u"control",{}).get(u"appearance") == constants.TABLE_LIST:
-                    begin_table_list = True
+                    table_list = True
                     new_json_dict[u"control"][u"appearance"] = u"field-list"
+                    #Generate a note label element so hints and labels
+                    #work as expected in table-lists.
+                    #see https://github.com/modilabs/pyxform/issues/62
+                    if 'label' in new_json_dict or 'hint' in new_json_dict:
+                        generated_label_element = {
+                            "type":"note",
+                            "name": "generated_table_list_label_" + str(row_number)
+                        }
+                        if 'label' in new_json_dict:
+                            generated_label_element[constants.LABEL] = new_json_dict[constants.LABEL]
+                            del new_json_dict[constants.LABEL]
+                        if 'hint' in new_json_dict:
+                            generated_label_element['hint'] = new_json_dict['hint']
+                            del new_json_dict['hint']
+                        child_list.append(generated_label_element)
                     
                 parent_children_array.append(new_json_dict)
                 stack.append((control_type, child_list))
@@ -478,10 +526,12 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
                 cascading_level = parse_dict["cascading_level"]
                 cascading_prefix = row.get(constants.NAME)
                 if not cascading_prefix:
-                    raise PyXFormError("Cascading select needs a name. Error on row: %s" % row_number)
-                cascading_json = get_cascading_json(cascading_choices, cascading_prefix, cascading_level)
-                
-                for c in cascading_json: parent_children_array.append(c)
+                    raise PyXFormError(rowFormatString % row_number + " Cascading select needs a name.")
+                #cascading_json = get_cascading_json(cascading_choices, cascading_prefix, cascading_level)
+                cascading_json = cascading_choices[0]['questions']
+                json_dict['choices'] = choices
+                for c in cascading_json:
+                    parent_children_array.append(c)
                 continue # so the row isn't put in as is
 
         #Try to parse question as a select:
@@ -493,43 +543,71 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
                 list_name = parse_dict["list_name"]
 
                 if list_name not in choices:
-                    raise PyXFormError("List name not in choices sheet: " + list_name + " Error on row: " + str(row_number))
+                    raise PyXFormError(rowFormatString % row_number + " List name not in choices sheet: " + list_name)
 
                 #Validate select_multiple choice names by making sure they have no spaces (will cause errors in exports).
                 if select_type == constants.SELECT_ALL_THAT_APPLY:
                     for choice in choices[list_name]:
                         if ' ' in choice[constants.NAME]:
-                                raise PyXFormError("Choice names with spaces cannot be added to multiple choice selects. See [" + choice[constants.NAME] + "] in [" + list_name + "]")
-
+                            raise PyXFormError("Choice names with spaces cannot be added to multiple choice selects. See [" + choice[constants.NAME] + "] in [" + list_name + "]")
+                
+                specify_other_question = None
                 if parse_dict.get("specify_other") is not None:
                     select_type += u" or specify other"
+#                    #With this code we no longer need to handle or_other questions in survey builder.
+#                    #However, it depends on being able to use choice filters and xpath expressions that return empty sets.
+#                    choices[list_name].append(
+#                        {
+#                            'name': 'other',
+#                            'label': {default_language : 'Other'},
+#                            'orOther': 'true',
+#                        })
+#                    or_other_xpath = 'isNull(orOther)'
+#                    if 'choice_filter' in row:
+#                        row['choice_filter'] += ' or ' + or_other_xpath
+#                    else:
+#                        row['choice_filter'] = or_other_xpath
+#                        
+#                    specify_other_question = \
+#                        {
+#                          'type':'text',
+#                          'name': row['name'] + '_specify_other',
+#                          'label':'Specify Other for:\n"' + row['label'] + '"',
+#                          'bind' : {'relevant': "selected(../%s, 'other')" % row['name']},
+#                        }
                     
                 new_json_dict = row.copy()
                 new_json_dict[constants.TYPE] = select_type
-                new_json_dict[constants.CHOICES] = choices[list_name]
+                new_json_dict['itemset'] = list_name
+                if row.get('choice_filter'):
+                    json_dict['choices'] = choices
+                else:
+                    new_json_dict[constants.CHOICES] = choices[list_name]
                 
                 #Code to deal with table_list appearance flags (for groups of selects)
-                if table_list or begin_table_list:
-                    if begin_table_list: #If this row is the first select in a table list
+                if table_list is not None:
+                    if not isinstance(table_list, basestring): #Then this row is the first select in a table list
                         table_list = list_name
                         table_list_header = {
                             constants.TYPE : select_type,
                             constants.NAME : "reserved_name_for_field_list_labels_" + str(row_number), #Adding row number for uniqueness
                             constants.CONTROL : { u"appearance" : u"label" },
-                            constants.CHOICES : choices[list_name]
+                            constants.CHOICES : choices[list_name],
+                            #Do we care about filtered selects in table lists?
+                            #'itemset' : list_name,
                         }
                         parent_children_array.append(table_list_header)
-                        begin_table_list = False
 
                     if table_list <> list_name:
-                        error_message = "Error on row: " + str(row_number) + "\n"
-                        error_message += "Badly formatted table list, list names don't match: " + table_list + " vs. " + list_name
+                        error_message = rowFormatString % row_number
+                        error_message += " Badly formatted table list, list names don't match: " + table_list + " vs. " + list_name
                         raise PyXFormError(error_message)
                     
                     control = new_json_dict[u"control"] = new_json_dict.get(u"control", {})
                     control[u"appearance"] = "list-nolabel"
-                        
                 parent_children_array.append(new_json_dict)
+                if specify_other_question:
+                    parent_children_array.append(specify_other_question)
                 continue
             
         #TODO: Consider adding some question_type validation here.
@@ -538,7 +616,32 @@ def workbook_to_json(workbook_dict, form_name=None, default_language=u"default",
         parent_children_array.append(row)
     
     if len(stack) != 1:
-        raise PyXFormError("unmatched begin statement: " + str(stack[-1][0]))
+        raise PyXFormError("Unmatched begin statement: " + str(stack[-1][0]))
+    
+    #Automatically add an instanceID element:
+    if yes_no_aliases.get(settings.get("omit_instanceID")):
+        if settings.get("public_key"):
+            raise PyXFormError("Cannot omit instanceID, it is required for encryption.")
+    else:
+        meta_element = \
+        {
+        "name":"meta",
+        "type":"group",
+        "control": {
+                    "bodyless": True
+                    },
+        "children": [{
+                    "name": "instanceID",
+                    "bind": {
+                             "readonly": "true()",
+                             "calculate": "concat('uuid:', uuid())"
+                             }, 
+                    "type": "calculate",
+                    }]
+        }
+        noop, survey_children_array = stack[0]
+        survey_children_array.append(meta_element)
+    
     #print_pyobj_to_json(json_dict)
     return json_dict
 
@@ -553,12 +656,10 @@ def parse_file_to_workbook_dict(path, file_object=None):
     (shortname, extension) = os.path.splitext(filename)
     if not extension: raise PyXFormError("No extension.")
     
-    if extension == ".xls":
+    if extension == ".xls" or extension == ".xlsx":
         return xls_to_dict(file_object if file_object is not None else path)
     elif extension == ".csv":
         return csv_to_dict(file_object if file_object is not None else path)
-    elif extension == ".xlsx":
-        raise PyXFormError("XLSX files are not supported at this time. Please save the spreadsheet as an XLS file (97).")
     else:
         raise PyXFormError("File was not recognized")
 
